﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMotorboat
{
    public class DrawningMotorboat
    {
        public EntityMotorboat? EntityMotorboat { get; private set; }

        private int? _pictureWidth;

        private int? _pictureHeight;

        private int? _startPosX;

        private int? _startPosY;
        private readonly int _drawningMotorboatWidth = 160;

        private readonly int _drawningMotorboatHeight = 60;

        public void Init(int speed, double weight, Color bodyColor, Color
        additionalColor, bool motor, bool glass)
        {
            EntityMotorboat = new EntityMotorboat();
            EntityMotorboat.Init(speed, weight, bodyColor, additionalColor,
            motor, glass);
            _pictureWidth = null;
            _pictureHeight = null;
            _startPosX = null;
            _startPosY = null;
        }
        public bool SetPictureSize(int width, int height)
        {
            if (_drawningMotorboatWidth < width && _drawningMotorboatHeight < height)
            {
            _pictureWidth = width;
            _pictureHeight = height;
                if (_startPosX.HasValue && _startPosY.HasValue)
                {
                    SetPosition(_startPosX.Value, _startPosY.Value);
                }

                return true;
            }
            
            return true;
        }

        public void SetPosition(int x, int y)
        {
            if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }

            if (x + _drawningMotorboatWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth - _drawningMotorboatWidth;
            }
            else if (x < 0)
            {
                _startPosX = 0;
            }
            else
            {
                _startPosX = x;
            }
            if (y + _drawningMotorboatHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight - _drawningMotorboatHeight;
            }
            else if (y < 0)
            {
                _startPosY = 0;
            }
            else
            {
                _startPosY = y;
            }
        }
        public bool MoveTransport(DirectionType direction)
        {
            if (EntityMotorboat == null || !_startPosX.HasValue ||
            !_startPosY.HasValue)
            {
                return false;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    if (_startPosX.Value - EntityMotorboat.Step > 0)
                    {
                        _startPosX -= (int)EntityMotorboat.Step;
                    }
                    return true;
                //вверх
                case DirectionType.Up:
                    if (_startPosY.Value - EntityMotorboat.Step > 0)
                    {
                        _startPosY -= (int)EntityMotorboat.Step;
                    }
                    return true;
                // вправо
                case DirectionType.Right:
                    if (_startPosX.Value + EntityMotorboat.Step + _drawningMotorboatWidth < _pictureWidth)
                    {
                        _startPosX += (int)EntityMotorboat.Step;
                    }
                    return true;
                //вниз
                case DirectionType.Down:
                    if (_startPosY.Value + EntityMotorboat.Step + _drawningMotorboatHeight < _pictureHeight)
                    {
                        _startPosY += (int)EntityMotorboat.Step;
                    }
                    return true;
                default:
                    return false;
            }
        }


        public void DrawTransport(Graphics g)
        {
            if (EntityMotorboat == null || !_startPosX.HasValue || !_startPosY.HasValue)
            { return; }
            Pen pen = new(Color.Black, 1);
            Brush mainBrush = new SolidBrush(EntityMotorboat.BodyColor);

            // корпус
            Point[] hull = new Point[]
            {
                new Point(_startPosX.Value + 5, _startPosY.Value + 0),
                new Point(_startPosX.Value + 120, _startPosY.Value + 0),
                new Point(_startPosX.Value + 160, _startPosY.Value + 35),
                new Point(_startPosX.Value + 120, _startPosY.Value + 70),
                new Point(_startPosX.Value + 5, _startPosY.Value + 70),
            };
            g.FillPolygon(mainBrush, hull);
            g.DrawPolygon(pen, hull);

            // стекло впереди
            if (EntityMotorboat.Glass) { 
                Brush glassBrush = new SolidBrush(Color.LightBlue);
                g.FillEllipse(glassBrush, _startPosX.Value + 20, _startPosY.Value + 15, 100, 40);
                g.DrawEllipse(pen, _startPosX.Value + 20, _startPosY.Value + 15, 100, 40);
            }

            // основная часть
            Brush blockBrush = new SolidBrush(EntityMotorboat.AdditionalColor);
            g.FillRectangle(blockBrush, _startPosX.Value + 20, _startPosY.Value + 15, 80, 40);
            g.DrawRectangle(pen, _startPosX.Value + 20, _startPosY.Value + 15, 80, 40);

            // двигатель 
            if (EntityMotorboat.Motor)
            {
                Brush engineBrush = new
                SolidBrush(EntityMotorboat.AdditionalColor);
                g.FillRectangle(engineBrush, _startPosX.Value + 0, _startPosY.Value + 10, 5, 50);
                g.DrawRectangle(pen, _startPosX.Value + 0, _startPosY.Value + 10, 5, 50);
            }
        }
    }
}



